# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from urllib.parse import urlparse

from openstack import proxy
from openstack.utils import urljoin

from otcextensions.sdk.dcs.v1 import backup as _backup
from otcextensions.sdk.dcs.v1 import config as _config
from otcextensions.sdk.dcs.v1 import instance as _instance
from otcextensions.sdk.dcs.v1 import restore_record as _restore_record
from otcextensions.sdk.dcs.v1 import statistic as _stat
from otcextensions.sdk.dcs.v1 import quota as _quota
from otcextensions.sdk.dcs.v1 import maintenance_time_window as _maintenance_tw
from otcextensions.sdk.dcs.v1 import service_specification as _service_spec
from otcextensions.sdk.dcs.v1 import availability_zone as _az


class Proxy(proxy.Proxy):

    skip_discovery = True

    def _get_endpoint_with_api_version(self):
        url_parts = urlparse(self.get_endpoint())
        api_version = url_parts.path.split('/').pop(1)
        alternate_endpoint = '{scheme}://{netloc}/{api_version}'.format(
            scheme=url_parts.scheme,
            netloc=url_parts.netloc,
            api_version=api_version
        )
        return alternate_endpoint

    # ======== Instances ========
    def create_instance(self, **kwargs):
        """Create an instance

        :param dict kwargs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        return self._create(_instance.Instance, **kwargs)

    def instances(self, **query):
        """List all cache instances

        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        return self._list(_instance.Instance, **query)

    def get_instance(self, instance):
        """Get detail about a given instance

        :param instance: The instance id or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: one object of class
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        return self._get(_instance.Instance, instance)

    def find_instance(self, name_or_id, ignore_missing=False):
        """Find instance by name or id

        :param name_or_id: The instance id or name of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: one object of class
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        return self._find(_instance.Instance, name_or_id,
                          ignore_missing=ignore_missing)

    def update_instance(self, instance, **attrs):
        """Update instance with attributes

        :param instance: The value can be the ID of an instance
            or a :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
            instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dcs.v1.instance.Instance`,
            comprised of the properties on the Instance class.
        :returns: The updated instance
        :rtype: :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        # Update method does not return the instance object which needs to be
        # fetched additionally in return statement.
        self._update(_instance.Instance, instance, **attrs)
        return self._get(_instance.Instance, instance)

    def delete_instance(self, instance, ignore_missing=True):
        """Delete an instance

        :param instance: The instance id, name or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the queue does not exist.
        :returns: `None`
        """
        self._delete(_instance.Instance, instance,
                     ignore_missing=ignore_missing)

    def extend_instance(self, instance, capacity):
        """Extend capacity of existing instance

        :param instance: The instance id, name or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :param int capacity: New instance capacity
        :returns: Updated instance
        :rtype: :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        res = self._get_resource(_instance.Instance, instance)
        res.extend(self, capacity)
        return self._get(_instance.Instance, res)

    def stop_instance(self, instance):
        """Stop existing instance

        :param instance: The instance id, name or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: Updated instance
        :rtype: :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        res = self.find_instance(instance)
        res.stop(self)
        return self._get(_instance.Instance, res)

    def start_instance(self, instance):
        """Start existing instance

        :param instance: The instance id, name or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: Updated instance
        :rtype: :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        res = self.find_instance(instance)
        res.start(self)
        return self._get(_instance.Instance, res)

    def restart_instance(self, instance):
        """Retart existing instance

        :param instance: The instance id, name or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: Updated instance
        :rtype: :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        res = self.find_instance(instance)
        res.restart(self)
        return self._get(_instance.Instance, res)

    def change_instance_password(self, instance,
                                 current_password, new_password):
        """Change instance password

        :param instance: The instance id, name or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :param current_password: Current instance password
        :param new_password: New instance password
        :returns: Updated instance
        :rtype: :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        """
        res = self.find_instance(instance)
        return res.change_pwd(
            self,
            current_password=current_password,
            new_password=new_password)

    # ======== Backups ========
    def backup_instance(self, instance, **kwargs):
        """Create an instance backup

        :param instance: The instance id or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :param dict kwargs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.dcs.v1.backup.Backup`
        """
        inst = self._get_resource(_instance.Instance, instance)
        return self._create(_backup.Backup, instance_id=inst.id, **kwargs)

    def backups(self, instance, **query):
        """List all instance backups

        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dcs.v1.backup.Backup`
        """
        inst = self._get_resource(_instance.Instance, instance)
        return self._list(
            _backup.Backup, paginated=False,
            instance_id=inst.id, **query)

    def delete_instance_backup(self, instance, backup, ignore_missing=True,
                               **attrs):
        """Delete an instance backup

        :param backup: The instance id, an instance of
            :class:`~otcextensions.sdk.dcs.v1.backup.Backup`
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the queue does not exist.
        :returns: `None`
        """
        inst = self._get_resource(_instance.Instance, instance)
        self._delete(_backup.Backup, backup, instance_id=inst.id,
                     ignore_missing=ignore_missing,
                     **attrs)

    # ======== Restores ========
    def restore_instance(self, instance, backup=None, **kwargs):
        """Restore instance from backup

        :param instance: The instance id or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :param dict kwargs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.dcs.v1.restore_record.RestoreRecord`
            `backup_id` and `description` are expected
        """
        inst = self._get_resource(_instance.Instance, instance)
        return self._create(
            _restore_record.RestoreRecord,
            instance_id=inst.id,
            **kwargs)

    def restore_records(self, instance, **query):
        """List all instance restore records

        :param instance: The instance id or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dcs.v1.restore_record.RestoreRecord`
        """
        inst = self._get_resource(_instance.Instance, instance)
        return self._list(
            _restore_record.RestoreRecord, paginated=False,
            instance_id=inst.id, **query)

    # ======== Misc ========
    def statistics(self):
        """Query statisctics for all instances

        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dcs.v1.stat.Statistics`
        """
        return self._list(_stat.Statistic, paginated=False)

    def instance_params(self, instance):
        """List all instance configuration records

        :param instance: The instance id or an instance of
            :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dcs.v1.config.Config`
        """
        inst = self._get_resource(_instance.Instance, instance)
        return self._list(
            _config.Config, paginated=False,
            instance_id=inst.id)

    def update_instance_params(self, instance, params):
        """Update instance configuration parameter with attributes

        :param instance: The value can be the ID of an instance
            or a :class:`~otcextensions.sdk.dcs.v1.instance.Instance`
            instance.
        :param params: List of parameters of
            a :class:`~otcextensions.sdk.dcs.v1.config.Config`.
        :returns: None
        """
        res = self._get_resource(_instance.Instance, instance)
        obj = self._get_resource(_config.Config, None, instance_id=res.id)
        return obj._update(
            self,
            params
        )

    # ======== Quotas ========
    def quotas(self):
        """Return a generator of quotas

        :returns: A generator of quota objects
        :rtype: :class:`~otcextensions.sdk.dcs.v1.quota.Quota`.
        """
        return self._list(_quota.Quota)

    # ======== Maintenance Time Window ========
    def maintenance_time_windows(self):
        """Return a generator of maintenance time windows

        :returns: A generator of maintenance time window objects
        :rtype:
            :class:`~sdk.dcs.v1.maintenance_time_window.MaintenanceTimeWindow`.
        """
        base = self._get_endpoint_with_api_version()
        base_path = urljoin(
            base, _maintenance_tw.MaintenanceTimeWindow.base_path
        )

        return self._list(
            _maintenance_tw.MaintenanceTimeWindow,
            base_path=base_path)

    # ======== Service Specification ========
    def service_specifications(self):
        """Return a generator of service specifications

        :returns: A generator of service specifications
        :rtype: :class:`~sdk.dcs.v1.service_specification.ServiceSpecification`
        """
        base = self._get_endpoint_with_api_version()
        base_path = urljoin(
            base, _service_spec.ServiceSpecification.base_path
        )

        return self._list(
            _service_spec.ServiceSpecification,
            base_path=base_path
        )

    # ========= Available Zone ========
    def availability_zones(self):
        """Return a generator of Availability Zones where a DCS instance
        resides.

        :returns: A generator of Availability Zone objects
        :rtype:
            :class:
                `~otcextensions.sdk.dcs.v1.availability_zone.AvailabilityZone`
        """
        base = self._get_endpoint_with_api_version()
        base_path = urljoin(base, _az.AvailabilityZone.base_path)

        return self._list(_az.AvailabilityZone, base_path=base_path)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Flavor(resource.Resource):
    resource_key = 'flavor'
    resources_key = 'flavors'
    base_path = '/elb/flavors'

    # capabilities
    allow_fetch = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'marker', 'limit', 'page_reverse',
        'id', 'name', 'type', 'shared'
    )

    # Properties
    #: Specifies the flavor ID.
    id = resource.Body('id')
    #: Specifies the flavor details.
    info = resource.Body('info', type=dict)
    #: Specifies whether the flavor is sold out.
    flavor_sold_out = resource.Body('flavor_sold_out', type=bool)
    #: Specifies the flavor name.
    name = resource.Body('name')
    #: Specifies whether the flavor is available to all users.
    shared = resource.Body('shared', type=bool)
    #: Specifies the project ID.
    project_id = resource.Body('project_id')
    #: Specifies the flavor type.
    type = resource.Body('type')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import backend_server_group

EXAMPLE_LABEL = {
    'label_name': 'env',
    'label_value': 'prod'
}

EXAMPLE_GROUP = {
    'gateway_id': 'gw-001',
    'vpc_channel_id': 'vpc-123',
    'member_group_name': 'backend-group',
    'member_group_remark': 'test group',
    'member_group_weight': 50,
    'dict_code': 'code-001',
    'microservice_version': '1.0.0',
    'microservice_port': 8080,
    'microservice_labels': [EXAMPLE_LABEL],
    'member_group_id': 'mg-123',
    'create_time': '2025-01-01T10:00:00Z',
    'update_time': '2025-01-02T10:00:00Z'
}


class TestBackendServerGroup(base.TestCase):

    def test_basic(self):
        sot = backend_server_group.BackendServerGroup()
        self.assertEqual(
            'apigw/instances/%(gateway_id)s/vpc-channels/'
            '%(vpc_channel_id)s/member-groups',
            sot.base_path
        )
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = backend_server_group.BackendServerGroup(**EXAMPLE_GROUP)
        self.assertEqual(EXAMPLE_GROUP['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_GROUP['vpc_channel_id'], sot.vpc_channel_id)
        self.assertEqual(EXAMPLE_GROUP['member_group_name'],
                         sot.member_group_name)
        self.assertEqual(EXAMPLE_GROUP['member_group_remark'],
                         sot.member_group_remark)
        self.assertEqual(EXAMPLE_GROUP['member_group_weight'],
                         sot.member_group_weight)
        self.assertEqual(EXAMPLE_GROUP['dict_code'], sot.dict_code)
        self.assertEqual(EXAMPLE_GROUP['microservice_version'],
                         sot.microservice_version)
        self.assertEqual(EXAMPLE_GROUP['microservice_port'],
                         sot.microservice_port)
        self.assertEqual(EXAMPLE_GROUP['member_group_id'],
                         sot.member_group_id)
        self.assertEqual(EXAMPLE_GROUP['create_time'], sot.create_time)
        self.assertEqual(EXAMPLE_GROUP['update_time'], sot.update_time)

        self.assertEqual(1, len(sot.microservice_labels))
        self.assertEqual('env', sot.microservice_labels[0].label_name)
        self.assertEqual('prod', sot.microservice_labels[0].label_value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk import sdk_resource

_logger = _log.setup_logging('openstack')


# class InstanceConfiguration(sdk_resource.Resource):
#     # TODO(agoncharov) most likely useless
#
#     base_path = '/%(project_id)s/instances/%(instanceId)s/configuration'
#     resource_key = 'instance'
#     service = rds_service.RdsService()
#
#     # capabilities
#     allow_get = True
#
#     # Properties
#     # Instance of the configuration
#     instanceId = resource.URI("instanceId")
#     #: Configuration list, key value pairs
#     #: *Type:dict*
#     configuration = resource.Body('configuration', type=dict)
#
#
# class Parameter(sdk_resource.Resource):
#     # TODO(agoncharov) most likely useless
#
#     base_path = '/%(project_id)s/datastores/versions/'
#     '%(datastore_version_id)s/parameters/'
#     resources_key = 'configuration-parameters'
#     service = rds_service.RdsService()
#
#     # capabilities
#     allow_list = True
#     allow_get = True
#
#     # Properties
#     #: Parameter name
#     name = resource.Body('name', alternate_id=True)
#     #: Minimum value of the parameter
#     #: *Type: int*
#     min = resource.Body('min', type=int)
#     #: Maximum value of the parameter
#     #: *Type: int*
#     max = resource.Body('max', type=int)
#     #: Parameter type
#     type = resource.Body('type')
#     #: Value range
#     value_range = resource.Body('value_range')
#     #: Description
#     description = resource.Body('description')
#     #: Require restart or not
#     #: *Type: bool*
#     restart_required = resource.Body('restart_required', type=bool)
#     #: Datastore version id
#     datastore_version_id = resource.Body('datastore_version_id')


class ConfigurationGroup(sdk_resource.Resource):

    base_path = '/%(project_id)s/configurations'
    resource_key = 'configuration'
    resources_key = 'configurations'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_update = True
    allow_get = True
    allow_list = True

    # Properties
    project_id = resource.URI('project_id')
    #: Id of the configuration group
    #: *Type:str*
    id = resource.Body('id')
    #: Name of the configuration group
    #: *Type:str*
    name = resource.Body('name')
    #: Description of the configuration group
    #: *Type:str*
    description = resource.Body('description')
    #: Id of Datastore version
    #: *Type:str*
    datastore_version_id = resource.Body('datastore_version_id')
    #: name of Datastore version
    #: *Type:str*
    datastore_version_name = resource.Body('datastore_version_name')
    #: name of Datastore
    #: *Type:str*
    datastore_name = resource.Body('datastore_name')
    #: Date of created
    #: *Type:str*
    created = resource.Body('created')
    #: Date of updated
    #: *Type:str*
    updated = resource.Body('updated')
    # : Allow update or not, 0 for not allowed
    # : *Type:int*
    # allowed_updated = resource.Body('allowed_updated', type=int)
    #: Count of associated instance
    #: *Type:int*
    instance_count = resource.Body('instance_count', type=int)
    #: Params values
    #: *Type:dict*
    values = resource.Body('values', type=dict)
    #: Parameter list
    #: *Type:dict*
    parameters = resource.Body('parameters', type=list)
    #: Datastore dict
    #: *Type:dict*
    datastore = resource.Body('datastore', type=dict)

    def get_associated_instances(self, session, endpoint_override=None):
        """Get associated instancs

        :param session: session (adapter)
        :param endpoint_override: optional endpoint_override

        :returns: list of instance id/name dicts with instances
            with this configuration group
        :rtype: list of dicts
        """

        request = self._prepare_request()
        session = self._get_session(session)

        if not endpoint_override:
            if getattr(self, 'endpoint_override', None):
                # If we have internal endpoint_override - use it
                endpoint_override = self.endpoint_override

        # Build additional arguments to the DELETE call
        args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers={'Content-Type': 'application/json'}
        )

        # URL is a subpoin
        url = utils.urljoin(request.url, 'instances')

        resp = session.get(url, **args)
        resp = resp.json()

        if resp:
            return resp['instances']

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        'DELETE' operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        # exceptions.raise_from_response(response, error_message=error_message)
        if has_body:
            body = response.json()

            errCode = body.get('errCode', None)
            if errCode and errCode == 'RDS.0041':
                if self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]

                body = self._consume_body_attrs(body)
                self._body.attributes.update(body)
                self._body.clean()
            elif errCode and errCode == 'RDS.0028':
                raise exceptions.NotFoundException('Resource not found')
            elif errCode and errCode != 'RDS.0041':
                _logger.error('error during service invokation %s' % errCode)
                raise exceptions.SDKException(
                    body.get('externalMessage', body)
                )
            elif not errCode:
                if self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]

                body = self._consume_body_attrs(body)
                self._body.attributes.update(body)
                self._body.clean()

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Topic(resource.Resource):
    resources_key = 'topics'
    base_path = '/notifications/topics'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'offset', 'limit')

    #: Resource identifier of a topic, which is unique
    id = resource.Body('topic_urn', alternate_id=True)
    #: Time when the topic was created
    #: The UTC time is in YYYY-MM-DDTHH:MM:SSZ format.
    create_time = resource.Body('create_time')
    #: Topic display name, which is presented as the name of
    #: the email sender in email messages
    #: Contains only digits, letters, underscores and hyphens
    display_name = resource.Body('display_name')
    #: Specifies the Topic Name.
    #: Contains only digits, letters, underscores and hyphens
    name = resource.Body('name')
    #: Message push policy
    #: 0: Failed messages will be saved in message queues.
    #: 1: Failed messages will be discarded.
    push_policy = resource.Body('push_policy', type=int)
    #: Unique Request ID
    request_id = resource.Body('request_id')
    #: Time when the topic was updated
    #: The UTC time is in YYYY-MM-DDTHH:MM:SSZ format.
    update_time = resource.Body('update_time')


class AttributeSpec(resource.Resource):
    #: Properties
    #: topic access policy
    access_policy = resource.Body('access_policy')
    #: Introduction
    introduction = resource.Body('introduction')


class TopicAttribute(resource.Resource):
    base_path = '/notifications/topics/%(topic_urn)s/attributes'

    allow_list = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'offset', 'limit', 'name')

    #: Properties
    #: Topic URN
    topic_urn = resource.URI('topic_urn')
    #: Unique Request ID
    request_id = resource.Body('request_id')
    #: Unique Request ID
    attributes = resource.Body('attributes', type=AttributeSpec)
    #: Values of topic attributes
    value = resource.Body('value')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.network.v2 import service_provider


class ServiceProvider(service_provider.ServiceProvider):
    @classmethod
    def list(cls, session, paginated=False,
             endpoint_override=None, headers=None, requests_auth=None,
             **params):
        """
        this dummy fake interface is temporarily introduced until
        demand OTCPR-24905 is completed
        """
        return [True]



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''SDRS Job CLI implementation'''
import logging
import json

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _add_sub_jobs_to_obj(obj, data, columns):
    name = 'entities'
    data += ('\n'.join((f'sub_job={ent.job_id}'
                        for ent in obj.entities.sub_jobs)),)

    columns = columns + (name,)
    return data, columns


def _add_server_group_to_obj(obj, data, columns):
    name = 'entities'
    data += ('protection_group_id=' + obj.entities.server_group_id,)

    columns = columns + (name,)
    return data, columns


def _add_parsed_task_to_obj(obj, data, columns):
    first, last = obj.fail_reason.split('error : ')
    error_dict = json.loads(last)
    message = next(iter(error_dict.values()))['message']
    code = next(iter(error_dict.values()))['code']
    data += (message,)
    columns = columns + ('message',)
    data += (code,)
    columns = columns + ('status_code',)
    data += (obj.error_code,)
    columns += ('error_code',)
    return data, columns


def _flatten_job(obj):
    """Flatten the structure of the job into a single dict
    """

    data = {
        'id': obj.job_id,
        'status': obj.status,
        'job_type': obj.job_type,
        'begin_time': obj.begin_time,
        'end_time': obj.end_time
    }

    return data


class ShowJob(command.ShowOne):
    _description = _('Show single job details')
    columns = (
        'id',
        'status',
        'job_type',
        'begin_time',
        'end_time'
    )

    def get_parser(self, prog_name):
        parser = super(ShowJob, self).get_parser(prog_name)
        parser.add_argument(
            'job',
            metavar='<job>',
            help=_('ID of the SDRS job.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.sdrs

        obj = client.get_job(
            job=parsed_args.job
        )

        data = utils.get_dict_properties(
            _flatten_job(obj), self.columns
        )

        if getattr(obj.entities, 'sub_jobs', None):
            data, self.columns = _add_sub_jobs_to_obj(obj, data,
                                                      self.columns)

        if getattr(obj.entities, 'server_group_id', None):
            data, self.columns = _add_server_group_to_obj(obj, data,
                                                          self.columns)

        if obj.fail_reason:
            data, self.columns = _add_parsed_task_to_obj(obj, data,
                                                         self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestApiCall(TestApiG):
    gateway = '560de602c9f74969a05ff01d401a53ed'

    def setUp(self):
        super(TestApiCall, self).setUp()

    def tearDown(self):
        super(TestApiCall, self).tearDown()

    def test_get_api_call_for_period(self):
        attrs = {
            'api_id': '64182cc7e77245ebbae8cf3b8522a540',
            'duration': '1h',
        }
        found = self.client.list_api_calls_for_period(gateway=self.gateway,
                                                      **attrs)
        self.assertIsNotNone(found)

    def test_get_api_call_for_group(self):
        attrs = {
            'group_id': 'ce973ff83ce54ef192c80bde884aa0ac',
        }
        found = self.client.list_api_calls_for_group(gateway=self.gateway,
                                                     **attrs)
        self.assertIsNotNone(found)

    def test_list_metric_data(self):
        attrs = {
            'dim': 'inbound_eip',
            'metric_name': 'upstream_bandwidth',
            'from': '1740787200000',
            'to': '1740873600000',
            'period': 3600,
            'filter': 'average',
        }
        found = self.client.list_metric_data(gateway=self.gateway, **attrs)
        self.assertIsNotNone(found)
        self.assertGreater(len(found), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import dependency

EXAMPLE = {
    "id": "4f4ae4eb-dcdc-4dd3-bffd-79600bd972b3",
    "owner": "*****",
    "link": "https://{bucket}.{obs_endpoint}.zip",
    "runtime": "Python3.6",
    "etag": "83863be4b6c3a86aef995dbc83aae68f",
    "size": 577118,
    "name": "python-kafka",
    "description": "Python library for Kafka operations.",
    "file_name": "python-kafka.zip",
    "version": 0,
    "dep_id": "edbd67fa-f107-40b3-af75-a85f0577ad61",
    "last_modified": 1660029887
}


class TestFunctionDependency(base.TestCase):

    def test_basic(self):
        sot = dependency.Dependency()
        path = '/fgs/dependencies'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = dependency.Dependency(**EXAMPLE)
        self.assertEqual(EXAMPLE['file_name'], sot.file_name)
        self.assertEqual(EXAMPLE['link'], sot.link)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log
from openstack import resource

from otcextensions.sdk.mrs.v1 import _base

_logger = _log.setup_logging('openstack')


class Datasource(_base.Resource):
    resource_key = 'data_source'
    resources_key = 'data_sources'
    base_path = '/data-sources'

    # capabilities
    allow_create = True
    allow_list = True
    allow_get = True
    allow_delete = True
    allow_update = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'type', 'url', 'description', 'is_public',
        'is_protected')

    #: Properties
    #: Data source ID
    id = resource.Body('id', alternate_id=True)
    #: Data source name
    name = resource.Body('name')
    #: type of the ds
    type = resource.Body('type')
    #: Data source URL
    url = resource.Body('url')
    #: Data source description
    description = resource.Body('description')
    #: Whether the data source is public
    is_public = resource.Body('is_public')
    #: Whether the data source is protected
    is_protected = resource.Body('is_protected')
    #: Data source creation time
    created_at = resource.Body('created_at')
    #: Data source update time
    updated_at = resource.Body('updated_at')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from unittest import mock

from openstackclient.tests.unit import utils

from otcextensions.osclient.load_balancer.v1 import pool_member
from otcextensions.tests.unit.osclient.load_balancer.v1 import fakes


class TestListPoolMember(fakes.TestLoadBalancer):

    _objects = fakes.FakePoolMember.create_multiple(3)

    columns = (
        'id', 'name', 'project_id', 'provisioning_status', 'address',
        'protocol_port', 'operating_status', 'weight')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.name,
            s.project_id,
            '',  # provisioning_status
            s.address,
            s.protocol_port,
            s.operating_status,
            s.weight,
        ))

    def setUp(self):
        super(TestListPoolMember, self).setUp()

        self.cmd = pool_member.ListPoolMember(self.app, None)

        self.client.pool_members = mock.Mock()

    def test_list_default(self):
        arglist = [
            'pool'
        ]

        verifylist = [
            ('pool', 'pool')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.pool_members.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.pool_members.assert_called_once_with(
            pool='pool'
        )

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))

    def test_list_filter_values(self):
        arglist = [
            'pool',
            '--name', 'name',
            '--protocol_port', '15',
            '--address', 'address',
            '--weight', '13',
        ]

        verifylist = [
            ('pool', 'pool'),
            ('name', 'name'),
            ('protocol_port', 15),
            ('address', 'address'),
            ('weight', 13),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.pool_members.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.pool_members.assert_called_once_with(
            pool='pool',
            address='address',
            name='name',
            protocol_port=15,
            weight=13
        )

    def test_list_filter_exceptions_proto_port(self):
        arglist = [
            'pool',
            '--protocol_port', '15c',
        ]

        verifylist = [
            ('protocol_port', 12)
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_list_filter_exceptions_weight(self):
        arglist = [
            'pool',
            '--weight', '15c',
        ]

        verifylist = [
            ('weight', 12)
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestShowPoolMember(fakes.TestLoadBalancer):

    _object = fakes.FakePoolMember.create_one()

    columns = (
        'address', 'admin_state_up', 'id', 'name',
        'operating_status',
        'protocol_port', 'subnet_id', 'weight')

    data = (
        _object.address,
        _object.is_admin_state_up,
        _object.id,
        _object.name,
        _object.operating_status,
        _object.protocol_port,
        _object.subnet_id,
        _object.weight,
    )

    def setUp(self):
        super(TestShowPoolMember, self).setUp()

        self.cmd = pool_member.ShowPoolMember(self.app, None)

        self.client.find_pool_member = mock.Mock()

    def test_show_default(self):
        arglist = [
            'pool_id',
            'member'
        ]

        verifylist = [
            ('pool', 'pool_id'),
            ('member', 'member')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_pool_member.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_pool_member.assert_called_once_with(
            name_or_id='member',
            pool='pool_id',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreatePoolMember(fakes.TestLoadBalancer):

    _object = fakes.FakePoolMember.create_one()

    columns = (
        'address', 'admin_state_up', 'id', 'name',
        'operating_status',
        'protocol_port', 'subnet_id', 'weight')

    data = (
        _object.address,
        _object.is_admin_state_up,
        _object.id,
        _object.name,
        _object.operating_status,
        _object.protocol_port,
        _object.subnet_id,
        _object.weight,
    )

    def setUp(self):
        super(TestCreatePoolMember, self).setUp()

        self.cmd = pool_member.CreatePoolMember(self.app, None)

        self.client.create_pool_member = mock.Mock()

    def test_create_default(self):
        arglist = [
            'pool_id',
            '--address', 'addr',
            '--protocol_port', '123',
            '--disable',
            '--name', 'name',
            '--subnet_id', 'subnet',
            '--weight', '13'
        ]

        verifylist = [
            ('pool', 'pool_id'),
            ('address', 'addr'),
            ('protocol_port', 123),
            ('disable', True),
            ('name', 'name'),
            ('subnet_id', 'subnet'),
            ('weight', 13),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_pool_member.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_pool_member.assert_called_once_with(
            address='addr',
            is_admin_state_up=False,
            name='name',
            pool='pool_id',
            protocol_port=123,
            subnet_id='subnet',
            weight=13
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_zero_weight(self):
        arglist = [
            'pool_id',
            '--address', 'addr',
            '--protocol_port', '123',
            '--disable',
            '--name', 'name',
            '--subnet_id', 'subnet',
            '--weight', '0'
        ]

        verifylist = [
            ('pool', 'pool_id'),
            ('address', 'addr'),
            ('protocol_port', 123),
            ('disable', True),
            ('name', 'name'),
            ('subnet_id', 'subnet'),
            ('weight', 0),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_pool_member.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_pool_member.assert_called_once_with(
            address='addr',
            is_admin_state_up=False,
            name='name',
            pool='pool_id',
            protocol_port=123,
            subnet_id='subnet',
            weight=0
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestUpdatePoolMember(fakes.TestLoadBalancer):

    _object = fakes.FakePoolMember.create_one()

    columns = (
        'address', 'admin_state_up', 'id', 'name',
        'operating_status',
        'protocol_port', 'subnet_id', 'weight')

    data = (
        _object.address,
        _object.is_admin_state_up,
        _object.id,
        _object.name,
        _object.operating_status,
        _object.protocol_port,
        _object.subnet_id,
        _object.weight,
    )

    def setUp(self):
        super(TestUpdatePoolMember, self).setUp()

        self.cmd = pool_member.SetPoolMember(self.app, None)

        self.client.update_pool_member = mock.Mock()

    def test_update_default(self):
        arglist = [
            'pool_id',
            'member',
            '--disable',
            '--name', 'name',
            '--weight', '13'
        ]

        verifylist = [
            ('pool', 'pool_id'),
            ('member', 'member'),
            ('disable', True),
            ('name', 'name'),
            ('weight', 13),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_pool_member.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_pool_member.assert_called_once_with(
            pool_member='member',
            is_admin_state_up=False,
            name='name',
            pool='pool_id',
            weight=13
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_update_zero_weight(self):
        arglist = [
            'pool_id',
            'member',
            '--disable',
            '--name', 'name',
            '--weight', '0'
        ]

        verifylist = [
            ('pool', 'pool_id'),
            ('member', 'member'),
            ('disable', True),
            ('name', 'name'),
            ('weight', 0),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_pool_member.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_pool_member.assert_called_once_with(
            pool_member='member',
            is_admin_state_up=False,
            name='name',
            pool='pool_id',
            weight=0
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestDeletePoolMember(fakes.TestLoadBalancer):

    _object = fakes.FakePoolMember.create_one()
    _pool = fakes.FakePool.create_one()

    def setUp(self):
        super(TestDeletePoolMember, self).setUp()

        self.cmd = pool_member.DeletePoolMember(self.app, None)

        self.client.delete_pool_member = mock.Mock()
        self.client.find_pool_member = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'pool_id',
            'member',
        ]

        verifylist = [
            ('pool', 'pool_id'),
            ('member', ['member']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_pool_member.side_effect = [
            {}
        ]
        self.client.find_pool_member.side_effect = [
            self._object
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_pool_member.assert_called_once_with(
            pool_member=self._object.id,
            pool='pool_id',
            ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions

from otcextensions.sdk import sdk_resource


class Resource(sdk_resource.Resource):

    def update(self, session, prepend_key=False, has_body=True,
               endpoint_override=None, headers=None, requests_auth=None):
        """Update the remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource update request.
                            Default to True.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_update` is not set to ``True``.
        """
        # The id cannot be dirty for an update
        self._body._dirty.discard("id")

        # Only try to update if we actually have anything to update.
        if not any([self._body.dirty, self._header.dirty]):
            return self

        if not self.allow_update:
            raise exceptions.MethodNotSupported(self, "update")

        request = self._prepare_request(prepend_key=prepend_key)
        session = self._get_session(session)

        args = self._prepare_override_args(
            endpoint_override=endpoint_override,
            request_headers=request.headers,
            additional_headers=headers,
            requests_auth=requests_auth)

        if self.commit_method == 'PATCH':
            response = session.patch(
                request.url, json=request.body, **args)
        elif self.commit_method == 'POST':
            response = session.post(
                request.url, json=request.body, **args)
        elif self.commit_method == 'PUT':
            response = session.put(
                request.url, json=request.body, **args)
        else:
            raise exceptions.ResourceFailure(
                message="Invalid update method: %s" % self.commit_method)

        self._translate_response(response, has_body=has_body)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack.tests.unit import base

from otcextensions.sdk.identity.v3 import custom_role

EXAMPLE = {
    "id": uuid.uuid4().hex,
    "name": "role" + uuid.uuid4().hex[:4],
    "description": "test",
    "domain_id": uuid.uuid4().hex,
    "catalog": "CUSTOMED",
    "display_name": "custom_" + uuid.uuid4().hex[:4],
    "type": "XA",
    "created_at": "1600181033358",
    "updated_at": "1600181033358",
}


class TestCustomRole(base.TestCase):

    def setUp(self):
        super(TestCustomRole, self).setUp()

    def test_basic(self):
        sot = custom_role.CustomRole()

        self.assertEqual('/v3.0/OS-ROLE/roles', sot.base_path)
        self.assertEqual('roles', sot.resources_key)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = custom_role.CustomRole(connection=self.cloud, **EXAMPLE)
        # Check how the override with "real" connection works
        self.assertEqual(
            'https://identity.example.com/v3.0/OS-ROLE/roles',
            sot.base_path)

        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import exceptions

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestKey(base.BaseFunctionalTest):
    KEY_ALIAS = 'sdkk_test_key'
    cmks = []

    def setUp(self):
        super(TestKey, self).setUp()
        try:
            self.cmk = self.conn.kms.create_key(
                key_alias=TestKey.KEY_ALIAS
            )
        except exceptions.DuplicateResource:
            self.cmk = self.conn.kms.find_key(alias=TestKey.KEY_ALIAS)

        self.cmks.append(self.cmk)

    def tearDown(self):
        try:
            for key in self.cmks:
                if key.id:
                    self.conn.kms.schedule_key_deletion(key, 7)
        except exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestKey, self).tearDown()

    def test_list(self):
        self.keys = list(self.conn.kms.keys())
        self.assertGreaterEqual(len(self.keys), 0)
        if len(self.keys) > 0:
            key = self.keys[0]
            k = self.conn.kms.get_key(key=key.id)
            self.assertIsNotNone(k)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.dns.v2 import _proxy
from otcextensions.sdk.dns.v2 import zone
from otcextensions.sdk.dns.v2 import nameserver as _ns
from otcextensions.sdk.dns.v2 import floating_ip
from otcextensions.sdk.dns.v2 import recordset

from openstack.tests.unit import test_proxy_base


class TestDnsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestDnsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestDnsZone(TestDnsProxy):
    def test_zone_create(self):
        self.verify_create(self.proxy.create_zone, zone.Zone,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id',
                                            'prepend_key': False})

    def test_zone_delete(self):
        self.verify_delete(self.proxy.delete_zone,
                           zone.Zone, True)

    def test_zone_find(self):
        self.verify_find(self.proxy.find_zone, zone.Zone)

    def test_zone_find_private(self):
        self.verify_find(self.proxy.find_zone, zone.Zone,
                         method_kwargs={
                             'p1': 'v1'
                         },
                         expected_kwargs={'p1': 'v1'})

    def test_zone_get(self):
        self.verify_get(self.proxy.get_zone, zone.Zone)

    def test_zones(self):
        self.verify_list(self.proxy.zones, zone.Zone)

    def test_zone_update(self):
        self.verify_update(self.proxy.update_zone, zone.Zone)

    def test_associate_router(self):
        self._verify(
            'otcextensions.sdk.dns.v2.zone.Zone.associate_router',
            self.proxy.add_router_to_zone,
            method_args=['zone_id'],
            method_kwargs={'x': 1},
            expected_args=[self.proxy],
            expected_kwargs={'x': 1}
        )

    def test_disassociate_router(self):
        self._verify(
            'otcextensions.sdk.dns.v2.zone.Zone.disassociate_router',
            self.proxy.remove_router_from_zone,
            method_args=['zone_id'],
            method_kwargs={'x': 1},
            expected_args=[self.proxy],
            expected_kwargs={'x': 1}
        )

    def test_ns(self):
        self.verify_list(
            self.proxy.nameservers, _ns.NameServer,
            method_kwargs={
                'zone': 'zone_id'
            },
            expected_kwargs={
                'paginated': False,
                'zone_id': 'zone_id'
            }
        )


class TestDnsRecordset(TestDnsProxy):
    def test_recordset_create(self):
        self.verify_create(self.proxy.create_recordset, recordset.Recordset,
                           method_kwargs={'zone': 'id'},
                           expected_kwargs={'zone_id': 'id',
                                            'prepend_key': False})

    def test_recordset_delete(self):
        self.verify_delete(self.proxy.delete_recordset,
                           recordset.Recordset, True)

    def test_recordset_update(self):
        self.verify_update(self.proxy.update_recordset, recordset.Recordset)

    def test_recordset_get(self):
        self.verify_get(self.proxy.get_recordset, recordset.Recordset,
                        method_kwargs={'zone': 'zid'},
                        expected_kwargs={'zone_id': 'zid'}
                        )

    def test_recordsets(self):
        self.verify_list(self.proxy.recordsets, recordset.Recordset,
                         base_path='/recordsets')

    def test_recordsets_zone(self):
        self.verify_list(self.proxy.recordsets, recordset.Recordset,
                         method_kwargs={'zone': 'zid'},
                         expected_kwargs={'zone_id': 'zid'})

    def test_recordset_find(self):
        self._verify("openstack.proxy.Proxy._find",
                     self.proxy.find_recordset,
                     method_args=['zone', 'rs'],
                     method_kwargs={},
                     expected_args=[recordset.Recordset, 'rs'],
                     expected_kwargs={'ignore_missing': True,
                                      'zone_id': 'zone'})


class TestDnsFloatIP(TestDnsProxy):
    def test_floating_ips(self):
        self.verify_list(self.proxy.floating_ips, floating_ip.FloatingIP)

    def test_floating_ip_get(self):
        self.verify_get(self.proxy.get_floating_ip, floating_ip.FloatingIP)

    def test_floating_ip_update(self):
        self.verify_update(self.proxy.update_floating_ip,
                           floating_ip.FloatingIP)

    def test_floating_ip_set(self):
        self.verify_update(self.proxy.set_floating_ip,
                           floating_ip.FloatingIP)

    def test_floating_ip_unset(self):
        self._verify('openstack.proxy.Proxy._update',
                     self.proxy.unset_floating_ip,
                     method_args=['value'],
                     method_kwargs={},
                     expected_args=[floating_ip.FloatingIP, 'value'],
                     expected_kwargs={'ptrdname': None})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json

from openstack import exceptions
from openstack import resource

from otcextensions.common import exc
from otcextensions.i18n import _
from otcextensions.sdk.auto_scaling.v1 import _base


class InstanceConfig(resource.Resource):
    #: Properties
    #: reserved property
    #: Server Instance reference Id, if set, auto-scaling-group will create
    #: new instance use the same config as this one.
    instance_id = resource.Body('instance_id')
    #: reserved property
    instance_name = resource.Body('instance_name')
    #: The flavor reference to be used
    flavor_id = resource.Body('flavorRef')
    #: The image reference to be used
    image_id = resource.Body('imageRef')
    #: The disks to be used
    disk = resource.Body('disk', type=list)
    #: The SSH key used to login server instance
    key_name = resource.Body('key_name')
    #: When a server is first created, it provides the administrator password.
    admin_password = resource.Body('adminPass')
    #: The file path and contents, text only, to inject into the server at
    #: launch. The maximum size of the file path data is 255 bytes.
    #: The maximum limit is The number of allowed bytes in the decoded,
    #: rather than encoded, data.
    personality = resource.Body('personality')
    #: EIP config for creating new instance
    public_ip = resource.Body('public_ip')
    #: reserved property, not used for now
    user_data = resource.Body('user_data')
    #: Metadata(key-pair) for creating new instance
    metadata = resource.Body('metadata', type=dict)
    #: Security groups
    security_groups = resource.Body('security_groups', type=list,
                                    list_type=dict)


class Config(_base.Resource):
    resource_key = 'scaling_configuration'
    resources_key = 'scaling_configurations'
    base_path = '/scaling_configuration'
    # query_marker_key = 'start_number'
    # service = auto_scaling_service.AutoScalingService()

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    # _query_mapping = resource.QueryParameters(
    #     'marker', 'limit',
    #     # name='scaling_configuration_name',
    #     marker=query_marker_key,
    #     limit='limit'
    # )

    #: Properties
    #: AutoScaling config ID
    id = resource.Body('scaling_configuration_id', alternate_id=True)
    #: AutoScaling config name
    name = resource.Body('scaling_configuration_name')
    #: AutoScaling config created time
    create_time = resource.Body('create_time')
    #: AutoScaling config status
    status = resource.Body('status')
    #: Use the exists instance as template to create new instance
    instance_config = resource.Body('instance_config',
                                    type=InstanceConfig)

    @classmethod
    def batch_delete(cls, session, configs):
        """Batch delete auto-scaling configs

        :param session: openstack session.
        :param configs: The list item value can contain IDs of a config
            or a :class:`~otcextensions.sdk.auto_scaling.v1.config.Config`
            instance.

        :returns: None
        """
        ids = [config.id if isinstance(config, Config) else config
               for config in configs]
        json_body = {'scaling_configuration_id': ids}
        response = session.post('/scaling_configurations',
                                headers={'Accept': '*'},
                                json=json_body)
        if response.status_code == 400:
            # Check if failed due to not exist
            content = response.json()
            error = content.get('error', None)
            if error.get('code', None) == 'AS.1038':
                ids = []
                message = error.get('message', None)
                if message.startswith('['):
                    items = json.loads(message)
                    for config_message in items:
                        id = config_message.get('id', None)
                        code = config_message.get('errorCode', None)
                        if code == 'AS.1004':
                            # Config does not exist
                            ids.append(id)
                if len(ids) > 0:
                    message = (_('AS Configurations (%(ids)s) not found') %
                               {'ids': ids})
                    raise exceptions.ResourceNotFound(message=message)
        # unknown failure
        exc.raise_from_response(response)
        return response



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.i18n import _

_logger = _log.setup_logging('openstack')


class Resource(resource.Resource):
    base_path = '/kms'

    #: Error code when create a secret key
    error_code = resource.Body('error_code')
    #: Error message when create a secret key
    error_msg = resource.Body('error_msg')

    def create(self, session, prepend_key=False, uri=None,
               requires_id=True, **params):
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        session = self._get_session(session)

        request = self._prepare_request(requires_id=False,
                                        prepend_key=prepend_key
                                        )
        # PATH is different
        if uri:
            request.url = uri
        elif self.create_path:
            request.url = self.create_path

        response = session.post(request.url,
                                json=request.body, headers=request.headers,
                                params=params)

        if response.status_code == 400:
            body = response.json()
            error = body.get('error', None)
            if error:
                if error['error_code'] == 'KMS.1104':
                    msg = (_('Key with given alias %s already exist')
                           % self.key_alias)
                    raise exceptions.DuplicateResource(msg)
                else:
                    msg = (_('Service returned an error with code=%s')
                           % error['error_code'])
                    raise exceptions.SDKException(msg)

        self._translate_response(response)

        return self

    def _action(self, session, url_part, body):
        """Perform actions given the message body.
        """
        url = utils.urljoin(self.base_path, url_part)
        response = session.post(
            url,
            json=body)
        self._translate_response(response)
        return self

    def _translate_response(
            self, response, has_body=None, error_message=None,
            resource_response_key=None
    ):
        """Given a KSA response, inflate this instance with its data

        'DELETE' operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        if response.status_code == 400 and not error_message:
            if has_body:
                body = response.json()
                error = body.get('error', None)
                if error:
                    error_code = error.get('error_code', None)
                    if error_code == 'KMS.0205':
                        raise exceptions.NotFoundException
                    else:
                        msg = (_('Service returned an error with code = %s')
                               % error_code)
                        raise exceptions.SDKException(msg)
        return super(Resource, self)._translate_response(
            response, has_body, error_message)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time
import calendar
from openstack import format


class YNBool(format.Formatter):
    @classmethod
    def deserialize(cls, value):
        """Convert a boolean string to a boolean"""
        if isinstance(value, bool):
            return value
        expr = str(value).lower()
        if "y" == expr:
            return True
        elif "n" == expr:
            return False
        else:
            raise ValueError("Unable to deserialize boolean string: %s"
                             % value)

    @classmethod
    def serialize(cls, value):
        """Convert a boolean to a boolean string"""
        if value in ["Y", "N", "y", "n"]:
            return str(value).upper()
        if isinstance(value, bool):
            if value:
                return "Y"
            else:
                return "N"
        else:
            raise ValueError("Unable to serialize boolean string: %s"
                             % value)


class Bool_10(format.Formatter):
    @classmethod
    def deserialize(cls, value):
        """Convert a boolean string to a boolean"""
        if isinstance(value, bool):
            return value
        expr = str(value).lower()
        if "1" == expr:
            return True
        elif "0" == expr:
            return False
        else:
            raise ValueError("Unable to deserialize boolean string: %s"
                             % value)

    @classmethod
    def serialize(cls, value):
        """Convert a boolean to a boolean string"""
        if value in ["1", "0"]:
            return str(value).upper()
        if isinstance(value, bool):
            if value:
                return "1"
            else:
                return "0"
        else:
            raise ValueError("Unable to serialize boolean string: %s"
                             % value)


class BoolStr_1(format.BoolStr):
    """Deserialize bool, which can be either bool or string
    """

    @classmethod
    def deserialize(cls, value):
        """Convert a boolean string to a boolean"""
        if isinstance(value, bool):
            return value
        expr = str(value).lower()
        if "true" == expr:
            return True
        elif "false" == expr:
            return False
        else:
            raise ValueError("Unable to deserialize boolean string: %s"
                             % value)


class ListRef(format.Formatter):
    """A formatter used to serialize/deserialize list reference

    [{"id": "any-id"}] <-> ["any-id"], for example.
    """

    @classmethod
    def deserialize(cls, value):
        """Convert a list primitive to list reference"""
        if isinstance(value, list):
            return [item["id"] for item in value]
        else:
            raise ValueError("Unable to deserialize list reference: %s"
                             % value)

    @classmethod
    def serialize(cls, value):
        """Convert list reference to list primitive"""
        if isinstance(value, list):
            return [{"id": item} for item in value]
        else:
            raise ValueError("Unable to serialize list reference: %s"
                             % value)


class TimeTMsStr(format.Formatter):

    @classmethod
    def deserialize(cls, value):
        """Convert a time_t with msec precision to ISO8601"""
        _time = time.gmtime(value / 1000)
        # Embed MS placeholder into the format string directly
        _format = '%Y-%m-%dT%H:%M:%S.{ms}+00:00'
        return time.strftime(_format, _time).format(
            ms=int(value % 1000))

    @classmethod
    def serialize(cls, value):
        """Convert ISO8601 to time_t with ms"""
        if isinstance(value, str):
            _time_t = None
            try:
                _time_t = time.strptime(value, '%Y-%m-%dT%H:%M:%S+00:00')
            except ValueError:
                _time_t = time.strptime(value, '%Y-%m-%dT%H:%M:%S')
            if _time_t:
                return calendar.timegm(_time_t) * 1000
            else:
                raise ValueError("Unable to parse time reference: %s"
                                 % value)
        elif isinstance(value, int):
            raise value
        else:
            raise ValueError("Unable to serialize list reference: %s"
                             % value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import time
import uuid

import fixtures
from openstack import _log
from openstack import exceptions
from openstack import utils

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestAs(base.BaseFunctionalTest):
    TIMEOUT = int(os.environ.get('OS_TEST_TIMEOUT'))
    UUID_V4 = uuid.uuid4().hex[:8]
    AS_GROUP = None
    AS_CONFIG = None
    AS_INSTANCE = None
    MAX_INST_NUMBER = 1
    IMAGE_NAME = "Standard_Debian_11_latest"
    DISK_SIZE = 4
    KP_NAME = None
    AS_GROUP_NAME = None
    NETWORK = None
    KEYPAIR = None

    def setUp(self):
        test_timeout = 3 * TestAs.TIMEOUT
        try:
            self.useFixture(
                fixtures.EnvironmentVariable(
                    'OS_TEST_TIMEOUT', str(test_timeout)))
        except ValueError:
            pass
        super(TestAs, self).setUp()
        self.client = self.conn.auto_scaling
        self.client_net = self.conn.network

    def create_network(
            self,
            cidr='192.168.0.0/16',
            ip_version=4,
            router_name='sdk-as-test-router-',
            net_name='sdk-as-test-net-',
            subnet_name='sdk-as-test-subnet-',
            kp_name='sdk-as-test-kp-'
    ):
        router_name += TestAs.UUID_V4
        net_name += TestAs.UUID_V4
        subnet_name += TestAs.UUID_V4
        kp_name += TestAs.UUID_V4
        if not TestAs.NETWORK:
            network = self.client_net.create_network(name=net_name)
            self.assertEqual(net_name, network.name)
            net_id = network.id
            subnet = self.client_net.create_subnet(
                name=subnet_name,
                ip_version=ip_version,
                network_id=net_id,
                cidr=cidr
            )
            self.assertEqual(subnet_name, subnet.name)
            subnet_id = subnet.id

            router = self.client_net.create_router(name=router_name)
            self.assertEqual(router_name, router.name)
            router_id = router.id
            interface = router.add_interface(
                self.conn.network,
                subnet_id=subnet_id
            )
            self.assertEqual(interface['subnet_id'], subnet_id)
            self.assertIn('port_id', interface)
            TestAs.NETWORK = {
                'router_id': router_id,
                'subnet_id': subnet_id,
                'network_id': net_id
            }
            if not TestAs.KEYPAIR:
                keypair = self.conn.compute.create_keypair(
                    name=kp_name
                )
                self.assertIsNotNone(keypair)
                TestAs.KEYPAIR = keypair
                TestAs.KP_NAME = kp_name
        return

    def destroy_network(self, params: dict, kp_name: str):
        router_id = params.get('router_id')
        subnet_id = params.get('subnet_id')
        network_id = params.get('network_id')

        keypair = self.conn.compute.delete_keypair(keypair=kp_name)
        self.assertIsNone(keypair)
        TestAs.KEYPAIR = None
        self.delete_router_interface_with_retry(router_id, subnet_id)
        self.delete_router_with_retry(router_id)
        sot = self.client_net.delete_subnet(
            subnet_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.client_net.delete_network(
            network_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)

    def get_image_id(self):
        image = self.conn.compute.find_image(
            name_or_id=self.IMAGE_NAME
        )
        if image:
            return image.id

    def get_default_sec_group(self):
        sec_group = self.client_net.find_security_group(
            name_or_id="default"
        )
        if sec_group:
            return sec_group.id

    def create_as_config(self, config_name='sdk-as-test-config-',
                         disk_vol_type="SATA", flavor="s3.medium.1",
                         disk_type="SYS", disk_size=4):
        if not TestAs.AS_CONFIG:
            config_name += TestAs.UUID_V4

            image_id = self.get_image_id()
            sec_group_id = self.get_default_sec_group()
            config_attrs = {
                "name": config_name,
                "instance_config": {
                    "flavorRef": flavor,
                    "imageRef": image_id,
                    "disk": [{
                        'size': disk_size,
                        'volume_type': disk_vol_type,
                        'disk_type': disk_type
                    }],
                    "key_name": TestAs.KP_NAME,
                    "security_groups": [{
                        "id": sec_group_id
                    }]
                }
            }
            as_config = self.client.create_config(**config_attrs)
            TestAs.AS_CONFIG = as_config

    def delete_as_config(self):
        if TestAs.AS_CONFIG:
            self.client.delete_config(config=TestAs.AS_CONFIG)
            TestAs.AS_CONFIG = None

    def create_as_group(self, group_name='test-as-group-',
                        desire_instances_number=1,
                        min_instances_number=0,
                        max_instances_number=MAX_INST_NUMBER):
        if not TestAs.AS_GROUP:
            group_name += TestAs.UUID_V4
            group_attrs = {
                "scaling_group_name": group_name,
                "scaling_configuration_id": TestAs.AS_CONFIG.id,
                "desire_instance_number": desire_instances_number,
                "min_instance_number": min_instances_number,
                "max_instance_number": max_instances_number,
                "vpc_id": TestAs.NETWORK.get('router_id'),
                "networks": [{
                    "id": TestAs.NETWORK.get('network_id')
                }]
            }
            as_group = self.client.create_group(**group_attrs)
            self.client.resume_group(as_group)
            as_group = self.client.wait_for_group(as_group)
            TestAs.AS_GROUP = as_group
            TestAs.AS_GROUP_NAME = group_name
            TestAs.MAX_INST_NUMBER = max_instances_number

    def delete_as_group(self, force_delete=False):
        if TestAs.AS_GROUP:
            as_group = TestAs.AS_GROUP
            self.client.pause_group(as_group)
            if not force_delete:
                self.delete_instances(as_group=as_group, timeout=self.TIMEOUT)
            self.client.delete_group(
                group=as_group,
                force_delete=force_delete
            )
            self.client.wait_for_delete_group(
                group=as_group,
                interval=5,
                wait=self.TIMEOUT
            )
            TestAs.AS_GROUP = None

    def delete_instances(self, as_group, timeout):
        try:
            instances = list(self.client.instances(group=as_group))
            if instances:
                for instance in instances:
                    self.client.remove_instance(
                        instance=instance,
                        delete_instance=True
                    )
                    self.client.wait_for_delete_instance(
                        instance=instance, wait=timeout
                    )
                return
        except exceptions.SDKException as e:
            _logger.warning(
                'Got exception during getting as instances %s' % e.message
            )

    def wait_for_instance(self):
        timeout = int(os.environ.get('OS_TEST_TIMEOUT'))
        for count in utils.iterate_timeout(
                timeout=timeout,
                message="Timeout waiting for instance"
        ):
            instances = list(self.client.instances(
                group=TestAs.AS_GROUP
            ))
            if len(instances) == TestAs.MAX_INST_NUMBER and instances[0].id:
                return self.client.wait_for_instance(instances[0])

    def delete_instance(self):
        if TestAs.AS_INSTANCE:
            self.client.remove_instance(
                instance=TestAs.AS_INSTANCE,
                delete_instance=True
            )
            self.client.wait_for_delete_instance(
                instance=TestAs.AS_INSTANCE,
                wait=self.TIMEOUT
            )

    def initialize_as_group_with_instance(self):
        self.create_as_config()
        self.create_as_group()
        TestAs.AS_INSTANCE = self.wait_for_instance()

    def deinitialize_as_group_with_instance(self):
        self.delete_instance()
        self.delete_as_group()
        self.delete_as_config()

    def retry(self, func, retries=3, delay=5):
        """Retry a function multiple times with a delay."""
        for attempt in range(retries):
            try:
                return func()
            except exceptions.ConflictException as e:
                if attempt == retries - 1:
                    raise e
                time.sleep(delay)

    def delete_router_with_retry(self, router_id):
        def delete_router():
            try:
                sot = self.client_net.delete_router(
                    router_id, ignore_missing=False
                )
                self.assertIsNone(sot)
            except exceptions.ConflictException as e:
                raise e

        self.retry(delete_router)

    def delete_router_interface_with_retry(self, router_id, subnet_id):
        def delete_interface():
            try:
                router = self.client_net.get_router(router_id)
                interface = router.remove_interface(
                    self.conn.network,
                    subnet_id=subnet_id
                )
                self.assertEqual(interface['subnet_id'], subnet_id)
                self.assertIn('port_id', interface)
            except exceptions.ConflictException as e:
                raise e

        self.retry(delete_interface)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils


class FailedTask(resource.Resource):
    """SDRS Failed task Resource"""
    resource_key = ''
    resources_key = 'failure_jobs'
    base_path = '/task-center/failure-jobs'

    # capabilities
    allow_create = False
    allow_list = True
    allow_fetch = False
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'failure_status', 'limit', 'marker', 'offset',
        'resource_name', 'resource_type', 'server_group_id')

    #: Properties
    #: Task operation time
    begin_time = resource.Body('begin_time')
    #: Failed task error code
    error_code = resource.Body('error_code')
    #: Failed task cause
    fail_reason = resource.Body('fail_reason')
    #: Failed task status
    failure_status = resource.Body('failure_status')
    #: Task ID
    job_id = resource.Body('job_id')
    #: Task name
    job_type = resource.Body('job_type')
    #: Task status
    #: Only 'fail' status available in current
    #: version
    job_status = resource.Body('job_status')
    #: Resource ID
    resource_id = resource.Body('resource_id')
    #: Resource name
    resource_name = resource.Body('resource_name')
    #: Resource type
    resource_type = resource.Body('resource_type')

    @classmethod
    def delete_all_tasks(cls, session, endpoint):
        """Method to delete all tasks of all protection groups

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param endpoint: SDRS service endpoint
        """
        url = utils.urljoin(endpoint, cls.base_path,
                            '/batch')
        return session.delete(url)

    @classmethod
    def delete_protection_tasks(cls, session,
                                endpoint, protection_group):
        """Method to delete all tasks of a signle protection group

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param endpoint: SDRS service endpoint
        :param protection_group: Protection group ID
        """
        url = utils.urljoin(endpoint, '/task-center', protection_group,
                            '/failure-jobs/batch')
        return session.delete(url)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
# import uuid
# from datetime import datetime
import mock

from osc_lib import utils as _osc_lib_utils

from openstackclient.tests.unit import utils

from otcextensions.tests.unit.osclient import test_base
from otcextensions.sdk.dis.v2 import stream
from otcextensions.sdk.dis.v2 import app
from otcextensions.sdk.dis.v2 import checkpoint
from otcextensions.sdk.dis.v2 import dump_task


def gen_data(obj, columns, formatters=None):
    """Fill expected data tuple based on columns list
    """
    return _osc_lib_utils.get_item_properties(
        obj, columns, formatters=formatters
    )


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(data.get(attr, '') for attr in columns)


class TestDis(utils.TestCommand):
    def setUp(self):
        super(TestDis, self).setUp()

        self.app.client_manager.dis = mock.Mock()

        self.client = self.app.client_manager.dis


class FakeStream(test_base.Fake):
    """Fake one or more Dis Streams."""
    @classmethod
    def generate(cls):
        """Create a fake DIS Stream.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            "auto_scale_enabled": True,
            "auto_scale_max_partition_count": 1,
            "auto_scale_min_partition_count": 1,
            "compression_format": "gzip",
            "create_time": 1687671774706,
            "data_type": "BLOB",
            "has_more_partitions": False,
            "last_modified_time": 1687671775413,
            "partitions": [
                {
                    "hash_range": "[0 : 461168601842738]",
                    "partition_id": "shardId-0000000000",
                    "sequence_number_range": "[0 : 0]",
                    "status": "ACTIVE"
                },
                {
                    "hash_range": "[461168601842738 : 922337203685477]",
                    "partition_id": "shardId-0000000001",
                    "sequence_number_range": "[0 : 0]",
                    "status": "ACTIVE"
                }
            ],
            "readable_partition_count": 2,
            "retention_period": 24,
            "status": "RUNNING",
            "stream_id": "VeamOHP5TNBWpprzcTi",
            "stream_name": "test-dis3",
            "stream_type": "COMMON",
            "writable_partition_count": 2
        }

        return stream.Stream(**object_info)


class FakeApp(test_base.Fake):
    """Fake one or more Dis Apps."""
    @classmethod
    def generate(cls):
        """Create a fake DIS App.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            "app_id": "bd6IPpvgiIflQPMpi9M",
            "app_name": "newstream",
            "create_time": 1593569685875,
            "commit_checkpoint_stream_names": ["newstream"]
        }

        return app.App(**object_info)


class FakeCheckpoint(test_base.Fake):
    """Fake one or more Dis Checkpoints."""
    @classmethod
    def generate(cls):
        """Create a fake DIS Checkpoint.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            "sequence_number": "newstram",
            "metadata": ""
        }
        return checkpoint.Checkpoint(**object_info)


class FakeDumpTask(test_base.Fake):
    """Fake one or more Dis Apps."""
    @classmethod
    def generate(cls):
        """Create a fake DIS App.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            "create_time": 1689377676849,
            "destination_type": "OBS",
            "exception_strategy": "ignoreAndBackup",
            "last_transfer_timestamp": 1689377695587,
            "obs_destination_description": {
                "agency_name": "dis_admin_agency",
                "consumer_strategy": "LATEST",
                "deliver_time_interval": 300,
                "destination_file_type": "text",
                "obs_bucket_path": "test-bucket-abcdef",
                "record_delimiter": "\n",
                "retry_duration": 0
            },
            "partitions": [
                {
                    "discard": 0,
                    "last_transfer_offset": 0,
                    "last_transfer_timestamp": 1689377695587,
                    "partitionId": "shardId-0000000000",
                    "state": "RUNNING"
                },
                {
                    "discard": 0,
                    "last_transfer_offset": 0,
                    "last_transfer_timestamp": 1689377695587,
                    "partitionId": "shardId-0000000001",
                    "state": "RUNNING"
                }
            ],
            "state": "RUNNING",
            "streamId": "bxdSOIEYPwJdSlAqq5z",
            "stream_id": "bxdSOIEYPwJdSlAqq5z",
            "stream_name": "test-dis",
            "task_id": "lJ4Ts5ObPd2UOpcRj3K",
            "task_name": "test-dump-task"
        }

        return dump_task.DumpTask(**object_info)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function
from otcextensions.sdk.function_graph.v2 import trigger

from openstack import _log

from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionTrigger(TestFg):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionTrigger, self).setUp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)

        self.trigger_attrs = {
            "trigger_type_code": "TIMER",
            "trigger_status": "ACTIVE",
            "event_data": {
                "name": "Timer-l8v2",
                "schedule": "3m",
                "schedule_type": "Rate"
            }
        }

        self.trigger = self.client.create_trigger(
            self.function, **self.trigger_attrs
        )
        assert isinstance(self.trigger, trigger.Trigger)

        self.addCleanup(
            self.client.delete_function,
            self.function
        )
        self.addCleanup(
            self.client.delete_trigger,
            self.function.func_urn,
            self.trigger_attrs["trigger_type_code"],
            self.trigger.trigger_id
        )

    def test_triggers(self):
        elist = list(self.client.triggers(
            function_urn=self.function.func_urn))
        self.assertIn(self.trigger["id"], elist[0].id)
        self.assertIn(
            self.trigger_attrs["event_data"]["schedule"],
            elist[0].event_data.schedule
        )

    def test_get_trigger(self):
        tr = self.client.get_trigger(
            self.function.func_urn,
            self.trigger_attrs["trigger_type_code"],
            self.trigger.trigger_id)
        self.assertIn(self.trigger.trigger_id, tr.trigger_id)

    def test_update_trigger(self):
        attrs = {
            "trigger_status": "DISABLED",
        }
        updated = self.client.update_trigger(
            self.function.func_urn,
            self.trigger_attrs["trigger_type_code"],
            self.trigger.trigger_id,
            **attrs
        )
        self.assertIn(
            attrs['trigger_status'],
            updated.trigger_status
        )

    # def test_delete_all_triggers(self):
    #     deleted = self.client.delete_all_triggers(
    #         self.function.func_urn,
    #     )
    #     self.assertIsNone(deleted)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ThrottlingPolicy(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/throttles'

    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True

    resources_key = 'throttles'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'name',
        'id', 'name', 'precise_search'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Maximum number of times the API can be accessed
    # by an app within the same period.
    app_call_limits = resource.Body('app_call_limits', type=int)
    # Request throttling policy name.
    name = resource.Body('name', type=str)
    # Time unit for limiting the number of API calls.
    # Enumeration values:
    # SECOND
    # MINUTE
    # HOUR
    # DAY
    time_unit = resource.Body('time_unit', type=str)
    # Description of the request throttling policy,
    # which can contain a maximum of 255 characters.
    remark = resource.Body('remark', type=str)
    # Maximum number of times an API can be accessed within a specified period.
    api_call_limits = resource.Body('api_call_limits', type=int)
    # Type of the request throttling policy.
    # 1: API-based, limiting the maximum number of times a
    # single API bound to the policy can be called within the specified period.
    # 2: API-shared, limiting the maximum number of times
    # all APIs bound to the policy can be called within the specified period.
    type = resource.Body('type', type=int)
    # Indicates whether to enable dynamic request throttling.
    # TRUE
    # FALSE
    enable_adaptive_control = resource.Body(
        'enable_adaptive_control', type=str
    )
    # Maximum number of times the API can be accessed
    # by a user within the same period.
    user_call_limits = resource.Body('user_call_limits', type=str)
    # Period of time for limiting the number of API calls.
    time_interval = resource.Body('time_interval', type=int)
    # Maximum number of times the API can be accessed
    # by an IP address within the same period.
    ip_call_limits = resource.Body('ip_call_limits', type=int)

    # Attributes
    # Number of APIs to which the request throttling policy has been bound.
    bind_num = resource.Body('bind_num', type=int)
    # Indicates whether an excluded request throttling
    # configuration has been created.
    # 1: yes
    # 2: no
    is_inclu_special_throttle = resource.Body(
        'is_inclu_special_throttle', type=int
    )
    # Creation time.
    created_at = resource.Body('create_time', type=str)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class DetailsSpec(resource.Resource):
    #: Specification (total memory) of the DCS instance.
    capacity = resource.Body('capacity')
    #: Maximum bandwidth supported by the specification.
    max_bandwidth = resource.Body('max_bandwidth')
    #: Maximum number of clients supported by the specification,
    # which is usually equal to the maximum number of connections.
    max_clients = resource.Body('max_clients')
    #: Maximum number of connections supported by the specification.
    max_connections = resource.Body('max_connections')
    #: Maximum inbound bandwidth supported by the specification,
    # which is usually equal to the maximum bandwidth.
    max_in_bandwidth = resource.Body('max_in_bandwidth')
    #: Maximum available memory.
    max_memory = resource.Body('max_memory')
    #: Number of tenant IP addresses corresponding to the specifications.
    tenant_ip_count = resource.Body('tenant_ip_count')
    #: Number of shards supported by the specifications.
    sharding_num = resource.Body('sharding_num')
    #: Number of proxies supported by Proxy Cluster instances of the specified
    # specifications. If the instance is not a Proxy Cluster instance, the
    # value of this parameter is 0.
    proxy_num = resource.Body('proxy_num')
    #: Number of DBs of the specifications.
    db_number = resource.Body('db_number')


class FlavorsSpec(resource.Resource):
    #: Specification (total memory) of the DCS instance.
    capacity = resource.Body('capacity')
    #: Memory unit.
    unit = resource.Body('unit')
    #: AZ ID.
    available_zones = resource.Body('available_zones', type=list)


class ServiceSpecification(resource.Resource):

    resources_key = 'products'

    base_path = '/products'

    # capabilities
    allow_list = True

    #: Properties
    #: Product ID used to differentiate DCS specifications.
    product_id = resource.Body('product_id')
    #: DCS instance specification code.
    spec_code = resource.Body('spec_code')
    #: DCS instance type.
    cache_mode = resource.Body('cache_mode')
    #: Edition of DCS for Redis.
    product_type = resource.Body('product_type')
    #: CPU architecture
    cpu_type = resource.Body('cpu_type')
    #: Storage type.
    storage_type = resource.Body('storage_type')
    #: Details of the specifications.
    details = resource.Body('details', type=DetailsSpec)
    #: Cache engine.
    engine = resource.Body('engine')
    #: Cache engine version.
    engine_versions = resource.Body('engine_versions')
    # DCS specifications. The value subjects to the returned specifications.
    spec_details = resource.Body('spec_details')
    #: Detailed DCS specifications, including the maximum number of
    # connections and maximum memory size.
    spec_details2 = resource.Body('spec_details2')
    #: Billing mode. Value: Hourly.
    charging_type = resource.Body('charging_type')
    #: Price of the DCS service to which you can subscribe.
    price = resource.Body('price')
    #: Currency
    currency = resource.Body('currency')
    #: Product type.
    prod_type = resource.Body('prod_type')
    #: Cloud service type code.
    cloud_service_type_code = resource.Body('cloud_service_type_code')
    #: Cloud resource type code.
    cloud_resource_type_code = resource.Body('cloud_resource_type_code')
    #: AZs with available resources.
    flavors = resource.Body('flavors', type=list, list_type=FlavorsSpec)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from otcextensions.tests.functional.osclient.nat.v2 import common


class TestSnat(common.NatTestCase):
    """Functional Tests for NAT Gateway"""

    def setUp(self):
        super(TestSnat, self).setUp()

    def test_snat_rule_list(self):
        json_output = json.loads(self.openstack(
            'nat snat rule list -f json '
        ))
        self.assertIsNotNone(json_output)

    def test_snat_rule_list_filters(self):
        json_output = json.loads(self.openstack(
            'nat snat rule list -f json '
            '--limit 1 --id 2 '
            '--project-id 3 '
            '--nat-gateway-id 4 '
            '--network-id 5 '
            '--cidr 6 '
            '--source-type 7 '
            '--floating-ip-id 8 '
            '--floating-ip-address 9 '
            '--status 10 '
            '--admin-state-up true '
            '--created-at "{}"'.format(self.CURR_TIME)
        ))
        self.assertIsNotNone(json_output)

    def test_nat_snat_rule(self):
        json_output = self.create_snat_rule()
        self.addCleanup(self.delete_snat_rule)
        snat_rule_id = json_output['id']
        nat_id = json_output['nat_gateway_id']

        # List Snat Rule by Snat Id filter
        json_output = json.loads(self.openstack(
            'nat snat rule list -f json '
            '--id ' + snat_rule_id
        ))
        self.assertIsNotNone(json_output)
        self.assertEqual(next(iter(json_output))['Id'], snat_rule_id)
        self.assertEqual(
            next(iter(json_output))['Nat Gateway Id'], nat_id)

        # List Snat Rule by nat-gateway-id filter
        json_output = json.loads(self.openstack(
            'nat snat rule list -f json '
            '--nat-gateway-id ' + nat_id
        ))
        self.assertIsNotNone(json_output)
        self.assertEqual(
            next(iter(json_output))['Nat Gateway Id'], nat_id)

        # Show Snat Rule by Id
        self.assertIsNotNone(self.SNAT_RULE_ID)
        json_output = json.loads(self.openstack(
            'nat snat rule show '
            ' -f json ' + self.SNAT_RULE_ID
        ))
        self.assertIsNotNone(json_output)
        self.assertEqual(json_output['id'], self.SNAT_RULE_ID)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.tests.functional.sdk.apig import TestApiG


class TestBackendServerGroup(TestApiG):
    gateway = "560de602c9f74969a05ff01d401a53ed"
    vpc = "80fc1bd30db94ef49468a27f536dd2cd"
    group = None

    def setUp(self):
        super(TestBackendServerGroup, self).setUp()
        attrs = {
            "member_groups": [{
                "member_group_name": "vpc_member_group",
                "member_group_weight": 10
            }]
        }
        result = self.client.add_or_update_backend_server_group(
            gateway=TestBackendServerGroup.gateway,
            vpc_channel=TestBackendServerGroup.vpc,
            **attrs
        )
        TestBackendServerGroup.group = result[0]
        self.assertEqual(TestBackendServerGroup.group.member_group_name,
                         "vpc_member_group")

    def tearDown(self):
        super(TestBackendServerGroup, self).tearDown()
        self.client.delete_backend_server_group(
            gateway=TestBackendServerGroup.gateway,
            vpc_channel=TestBackendServerGroup.vpc,
            backend_group=TestBackendServerGroup.group
        )

    def test_01_list_backend_server_groups(self):
        found = list(self.client.backend_server_groups(
            gateway=TestBackendServerGroup.gateway,
            vpc_channel=TestBackendServerGroup.vpc))
        self.assertEqual(len(found), 1)

    def test_02_update_backend_server_group(self):
        attrs = {
            "member_group_name": "vpc_member_group",
            "member_group_weight": 50
        }
        updated = self.client.update_backend_server_group(
            gateway=TestBackendServerGroup.gateway,
            vpc_channel=TestBackendServerGroup.vpc,
            backend_group=TestBackendServerGroup.group,
            **attrs
        )
        self.assertEqual(updated.member_group_weight,
                         attrs.get("member_group_weight"))

    def test_03_get_backend_server_group(self):
        found = self.client.get_backend_server_group(
            gateway=TestBackendServerGroup.gateway,
            vpc_channel=TestBackendServerGroup.vpc,
            backend_group=TestBackendServerGroup.group)
        self.assertEqual(found.member_group_id,
                         TestBackendServerGroup.group.member_group_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import listener

EXAMPLE = {
    'admin_state_up': True,
    'client_ca_tls_container_ref': 'client-ca-tls-container-ref',
    'client_timeout': 80,
    'created_at': 'created-at',
    'connection_limit': 80,
    'description': 'description',
    'default_pool_id': 'default-pool-id',
    'default_tls_container_ref': 'default-tls-container-ref',
    'enable_member_retry': True,
    'enhance_l7policy_enable': True,
    'http2_enable': True,
    'insert_headers': {'X-Forwarded-ELB-IP': True},
    'loadbalancers': [{'id': 'lb-uuid1'}, {'id': 'lb-uuid2'}],
    'loadbalancer_id': 'loadbalancer_id',
    'ipgroup': {'ipgroup_id': 'ipgroup-id',
                'enable_ipgroup': True,
                'type': 'type'},
    'name': 'name',
    'keepalive_timeout': 5,
    'member_timeout': 5,
    'protocol': 'protocol',
    'protocol_port': 5,
    'project_id': 'project-id',
    'security_policy_id': 'security-policy-id',
    'sni_container_refs': [],
    'sni_match_algo': 'sni-match-algo',
    'transparent_client_ip_enable': True,
    'tls_ciphers_policy': 'tls-ciphers-policy',
    'updated_at': 'updated-at',
    'tags': [{
        "key": "test",
        "value": "api"
    }]
}


class TestListener(base.TestCase):

    def test_basic(self):
        sot = listener.Listener()
        path = '/elb/listeners'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = listener.Listener(**EXAMPLE)
        self.assertEqual(EXAMPLE['admin_state_up'],
                         sot.is_admin_state_up)
        self.assertEqual(EXAMPLE['client_ca_tls_container_ref'],
                         sot.client_ca_tls_container_ref)
        self.assertEqual(EXAMPLE['client_timeout'], sot.client_timeout)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['connection_limit'], sot.connection_limit)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['default_pool_id'], sot.default_pool_id)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['default_tls_container_ref'],
                         sot.default_tls_container_ref)
        self.assertEqual(EXAMPLE['enable_member_retry'],
                         sot.enable_member_retry)
        self.assertEqual(EXAMPLE['enhance_l7policy_enable'],
                         sot.enhance_l7policy)
        self.assertEqual(EXAMPLE['http2_enable'], sot.http2_enable)
        self.assertEqual(EXAMPLE['insert_headers'], sot.insert_headers)
        self.assertEqual(EXAMPLE['loadbalancer_id'], sot.loadbalancer_id)
        self.assertEqual(EXAMPLE['loadbalancers'], sot.load_balancers)
        self.assertEqual(EXAMPLE['ipgroup'], sot.ipgroup)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['keepalive_timeout'], sot.keepalive_timeout)
        self.assertEqual(EXAMPLE['member_timeout'], sot.member_timeout)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['protocol_port'], sot.protocol_port)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['security_policy_id'], sot.security_policy_id)
        self.assertEqual(EXAMPLE['sni_container_refs'], sot.sni_container_refs)
        self.assertEqual(EXAMPLE['sni_match_algo'], sot.sni_match_algo)
        self.assertEqual(EXAMPLE['tags'], sot.tags)
        self.assertEqual(EXAMPLE['transparent_client_ip_enable'],
                         sot.transparent_client_ip_enable)
        self.assertEqual(EXAMPLE['tls_ciphers_policy'], sot.tls_ciphers_policy)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.bms.v1 import _proxy


class TestBMSProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestBMSProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestExtractName(TestBMSProxy):

    def test_extract_name(self):

        self.assertEqual(
            [],
            self.proxy._extract_name(
                '/v1',
                project_id='7ed5f793b8354ea9b27a849f17af4733')
        )

        self.assertEqual(
            ['flavors'],
            self.proxy._extract_name(
                '/v1/7ed5f793b8354ea9b27a849f17af4733/flavors',
                project_id='7ed5f793b8354ea9b27a849f17af4733')
        )
        self.assertEqual(
            ['detail'],
            self.proxy._extract_name(
                '/v1/7ed5f793b8354ea9b27a849f17af4733/detail',
                project_id='7ed5f793b8354ea9b27a849f17af4733')
        )
        self.assertEqual(
            ['limits'],
            self.proxy._extract_name(
                '/v1/7ed5f793b8354ea9b27a849f17af4733/limits',
                project_id='7ed5f793b8354ea9b27a849f17af4733')
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.swr.v2 import repository

EXAMPLE = {
    'namespace': 'org_name',
    'repository': 'repo_name',
    'category': 'windows',
    'description': 'desc',
    'is_public': False,
}

EXAMPLE_PERMISSION = {
    'permissions': [
        {
            'user_id': '5a23ecb3999b458d92d51d524bb7fb4b',
            'user_name': 'test',
            'user_auth': 1
        }
    ],
    'namespace': 'test_create_org_v2',
    'repository': 'repo_name',
}


class TestRepository(base.TestCase):

    def test_basic(self):
        sot = repository.Repository()
        path = '/manage/namespaces/%(namespace)s/repos'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = repository.Repository(**EXAMPLE)
        self.assertEqual(EXAMPLE['namespace'], sot.namespace)
        self.assertEqual(EXAMPLE['repository'], sot.repository)
        self.assertEqual(EXAMPLE['category'], sot.category)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['is_public'], sot.is_public)


class TestRepositoryPermissions(base.TestCase):

    def test_basic(self):
        sot = repository.Permission()
        path = '/manage/namespaces/%(namespace)s/repos/%(repository)s/access'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = repository.Permission(**EXAMPLE_PERMISSION)
        self.assertEqual(EXAMPLE_PERMISSION['namespace'], sot.namespace)
        self.assertEqual(
            EXAMPLE_PERMISSION['permissions'][0]['user_id'],
            sot.permissions[0].user_id
        )
        self.assertEqual(
            EXAMPLE_PERMISSION['permissions'][0]['user_auth'],
            sot.permissions[0].user_auth
        )
        self.assertEqual(
            EXAMPLE_PERMISSION['permissions'][0]['user_name'],
            sot.permissions[0].user_name
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import apienvironment as env

EXAMPLE_ENVIRONMENT = {
    'name': 'test-environment',
    'id': 'env-12345',
    'remark': 'This is a test environment',
    'create_time': '2024-02-05T12:34:56Z',
}


class TestApiEnvironment(base.TestCase):

    def test_basic(self):
        sot = env.ApiEnvironment()
        self.assertEqual('/apigw/instances/%(gateway_id)s/envs',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('envs', sot.resources_key)

    def test_make_it(self):
        sot = env.ApiEnvironment(**EXAMPLE_ENVIRONMENT)
        self.assertEqual(EXAMPLE_ENVIRONMENT['name'], sot.name)
        self.assertEqual(EXAMPLE_ENVIRONMENT['id'], sot.id)
        self.assertEqual(EXAMPLE_ENVIRONMENT['remark'], sot.remark)
        self.assertEqual(EXAMPLE_ENVIRONMENT['create_time'], sot.create_time)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.dns.v2 import recordset
from otcextensions.tests.unit.osclient.dns.v2 import fakes


class TestListRS(fakes.TestDNS):

    objects = fakes.FakeRecordset.create_multiple(3)
    _zone = fakes.FakeZone.create_one()

    columns = (
        'id', 'name', 'type', 'status', 'description', 'records'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListRS, self).setUp()

        self.cmd = recordset.ListRS(self.app, None)

        self.client.recordsets = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.recordsets

    def test_default_zone(self):
        arglist = [
            'zn'
        ]

        verifylist = [
            ('zone', 'zn')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]
        self.client.find_zone.side_effect = [
            self._zone
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False,
        )
        self.client.api_mock.assert_called_once_with(
            zone=self._zone
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_private_zone(self):
        arglist = [
            'zn',
            '--zone-type', 'private'
        ]

        verifylist = [
            ('zone', 'zn'),
            ('zone_type', 'private')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]
        self.client.find_zone.side_effect = [
            self._zone
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            zone_type='private',
            ignore_missing=False,
        )
        self.client.api_mock.assert_called_once_with(
            zone=self._zone
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowRS(fakes.TestDNS):

    _data = fakes.FakeRecordset.create_one()
    _zone = fakes.FakeZone.create_one()

    columns = (
        'description', 'name', 'records', 'status', 'ttl', 'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowRS, self).setUp()

        self.cmd = recordset.ShowRS(self.app, None)

        self.client.find_zone = mock.Mock()
        self.client.find_recordset = mock.Mock()
        self.client.api_mock = self.client.find_recordset

    def test_default(self):
        arglist = [
            'zone',
            'rs'
        ]

        verifylist = [
            ('zone', 'zone'),
            ('recordset', 'rs')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zone',
            ignore_missing=False,
            zone_type=None
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._zone,
            name_or_id='rs'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_private(self):
        arglist = [
            'zone',
            'rs',
            '--zone-type', 'private'
        ]

        verifylist = [
            ('zone', 'zone'),
            ('recordset', 'rs'),
            ('zone_type', 'private')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zone',
            ignore_missing=False,
            zone_type='private'
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._zone,
            name_or_id='rs'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateRS(fakes.TestDNS):

    _data = fakes.FakeRecordset.create_one()
    _zone = fakes.FakeZone.create_one()

    columns = (
        'description', 'name', 'records', 'status', 'ttl', 'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateRS, self).setUp()

        self.cmd = recordset.CreateRS(self.app, None)

        self.client.create_recordset = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.api_mock = self.client.create_recordset

    def test_create(self):
        arglist = [
            'zn',
            '--name', 'rs',
            '--description', 'descr',
            '--type', 'A',
            '--ttl', '500',
            '--record', 'a=b',
            '--record', 'c=d',
        ]

        verifylist = [
            ('zone', 'zn'),
            ('name', 'rs'),
            ('description', 'descr'),
            ('type', 'A'),
            ('ttl', 500),
            ('record', ['a=b', 'c=d']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            zone=self._zone,
            description='descr',
            name='rs',
            type='A',
            ttl=500,
            records=['a=b', 'c=d']
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_private(self):
        arglist = [
            'zn',
            '--name', 'rs',
            '--description', 'descr',
            '--type', 'A',
            '--ttl', '500',
            '--record', 'a=b',
            '--record', 'c=d',
            '--zone-type', 'private'
        ]

        verifylist = [
            ('zone', 'zn'),
            ('name', 'rs'),
            ('description', 'descr'),
            ('type', 'A'),
            ('ttl', 500),
            ('record', ['a=b', 'c=d']),
            ('zone_type', 'private')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False,
            zone_type='private'
        )

        self.client.api_mock.assert_called_once_with(
            zone=self._zone,
            description='descr',
            name='rs',
            type='A',
            ttl=500,
            records=['a=b', 'c=d']
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetRS(fakes.TestDNS):

    _data = fakes.FakeRecordset.create_one()
    _zone = fakes.FakeZone.create_one()

    columns = (
        'description', 'name', 'records', 'status', 'ttl', 'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestSetRS, self).setUp()

        self.cmd = recordset.SetRS(self.app, None)

        self.client.update_recordset = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.find_recordset = mock.Mock()
        self.client.api_mock = self.client.update_recordset

    def test_set(self):
        arglist = [
            'zn',
            'rs',
            '--description', 'descr',
            '--ttl', '500',
            '--record', 'a=b',
            '--record', 'c=d',
        ]

        verifylist = [
            ('zone', 'zn'),
            ('recordset', 'rs'),
            ('description', 'descr'),
            ('ttl', 500),
            ('record', ['a=b', 'c=d']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]
        self.client.api_mock.side_effect = [
            self._data
        ]
        self.client.find_recordset.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_recordset.assert_called_with(zone=self._zone,
                                                      name_or_id='rs')

        self.client.api_mock.assert_called_once_with(
            recordset=self._data,
            description='descr',
            records=['a=b', 'c=d'],
            ttl=500,
            zone_id=self._zone.id
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteRS(fakes.TestDNS):
    _zone = fakes.FakeZone.create_one()
    _rs = fakes.FakeRecordset.create_one()

    def setUp(self):
        super(TestDeleteRS, self).setUp()

        self.cmd = recordset.DeleteRS(self.app, None)

        self.client.delete_recordset = mock.Mock()
        self.client.find_zone = mock.Mock()
        self.client.find_recordset = mock.Mock()
        self.client.api_mock = self.client.delete_recordset

    def test_delete_multiple(self):
        arglist = [
            'zn',
            't1',
            't2',
        ]
        verifylist = [
            ('zone', 'zn'),
            ('recordset', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]

        self.client.find_recordset.side_effect = [self._rs, self._rs]
        self.client.api_mock.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        find_calls = [
            mock.call(zone=self._zone, name_or_id='t1', ignore_missing=False),
            mock.call(zone=self._zone, name_or_id='t2', ignore_missing=False)
        ]

        self.client.find_recordset.assert_has_calls(find_calls)

        calls = [
            mock.call(zone=self._zone, recordset=self._rs,
                      ignore_missing=False),
            mock.call(zone=self._zone, recordset=self._rs,
                      ignore_missing=False)
        ]

        self.client.api_mock.assert_has_calls(calls)
        self.assertEqual(2, self.client.api_mock.call_count)

    def test_private(self):
        arglist = [
            'zn',
            't1',
            '--zone-type', 'private'
        ]
        verifylist = [
            ('zone', 'zn'),
            ('recordset', ['t1']),
            ('zone_type', 'private')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_zone.side_effect = [
            self._zone
        ]

        self.client.find_recordset.side_effect = [self._rs, self._rs]
        self.client.api_mock.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_zone.assert_called_once_with(
            'zn',
            ignore_missing=False,
            zone_type='private'
        )

        find_calls = [
            mock.call(zone=self._zone, name_or_id='t1', ignore_missing=False),
        ]

        self.client.find_recordset.assert_has_calls(find_calls)

        calls = [
            mock.call(zone=self._zone, recordset=self._rs,
                      ignore_missing=False),
        ]

        self.client.api_mock.assert_has_calls(calls)
        self.assertEqual(1, self.client.api_mock.call_count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dcaas.v2 import endpoint_group


EXAMPLE = {
    "name": "endpoint group1",
    "description": "test description",
    "endpoints": ["10.2.0.0/24", "10.3.0.0/24"],
    "tenant_id": "6fbe9263116a4b68818cf1edce16bc4f",
    "type": "cidr"
}


class TestDirectConnectEndpointGroup(base.TestCase):

    def setUp(self):
        super(TestDirectConnectEndpointGroup, self).setUp()
        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.put = mock.Mock()

    def test_basic(self):
        sot = endpoint_group.DirectConnectEndpointGroup()
        self.assertEqual('/dcaas/dc-endpoint-groups', sot.base_path)
        self.assertEqual('dc_endpoint_group', sot.resource_key)
        self.assertEqual('dc_endpoint_groups', sot.resources_key)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = endpoint_group.DirectConnectEndpointGroup(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['endpoints'], sot.endpoints)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['type'], sot.type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import _log
from openstack import exceptions

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestCleanup(base.BaseFunctionalTest):
    def setUp(self):
        super(TestCleanup, self).setUp()

    def test_01_create(self):
        flavor = list(self.conn.dws.flavors())[0]
        network = list(self.conn.network.networks())[0]
        router = list(self.conn.network.routers())[0]
        security_group = list(self.conn.network.security_groups())[0]
        attrs = {
            'name': 'dws-1',
            'flavor': flavor['name'],
            'num_nodes': 3,
            'availability_zone': 'eu-de-01',
            'router_id': router['id'],
            'network_id': network['id'],
            'security_group_id': security_group['id'],
            'port': 8000,
            'user_name': 'dbadmin',
            'user_pwd': 'PasswordDbGauss!@',
            'public_ip': {
                'public_bind_type': 'auto_assign',
                'eip_id': ''
            }
        }
        self.conn.dws.create_cluster(**attrs)
        try:
            resource.wait_for_status(
                self.client, self.client.get_cluster(self.cluster.id),
                "AVAILABLE",
                ["CREATION FAILED", "UNAVAILABLE"],
                10,
                1200
            )
        except exceptions.ResourceFailure:
            self.client.delete_cluster(self.cluster.id)

    def test_02_delete(self):
        clusters = list(self.conn.dws.clusters())
        self.conn.dws._service_cleanup(dry_run=False)
        clusters = list(self.conn.dws.clusters())
        self.assertEqual(len(clusters), 0)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.dms.v1 import group
from otcextensions.tests.unit.osclient.dms.v1 import fakes


class TestGroup(fakes.TestDMS):

    def setUp(self):
        super(TestGroup, self).setUp()
        self.client = self.app.client_manager.dms


class TestListGroup(TestGroup):

    groups = fakes.FakeGroup.create_multiple(3)
    queue = fakes.FakeQueue.create_one()

    columns = ('ID', 'name', 'produced_messages', 'consumed_messages',
               'available_messages')
    columns_with_dead = ('ID', 'name', 'produced_messages',
                         'consumed_messages', 'available_messages',
                         'produced_deadletters', 'available_deadletters')

    data = []
    data_with_dead = []

    for s in groups:
        data.append((
            s.id,
            s.name,
            s.produced_messages,
            s.consumed_messages,
            s.available_messages,
        ))
        data_with_dead.append((
            s.id,
            s.name,
            s.produced_messages,
            s.consumed_messages,
            s.available_messages,
            s.produced_deadletters,
            s.available_deadletters,
        ))

    def setUp(self):
        super(TestListGroup, self).setUp()

        self.cmd = group.ListGroup(self.app, None)

        self.client.groups = mock.Mock()
        self.client.find_queue = mock.Mock()

    def test_list_group(self):
        arglist = [
            'queue_id'
        ]

        verifylist = [
            ('queue', 'queue_id'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.groups.side_effect = [
            self.groups
        ]
        self.client.find_queue.side_effect = [
            self.queue
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.groups.assert_called_once_with(
            queue=self.queue.id,
            include_deadletter=False)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_group_ext(self):
        arglist = [
            'queue_id',
            '--include_deadletter'
        ]

        verifylist = [
            ('queue', 'queue_id'),
            ('include_deadletter', True)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.groups.side_effect = [
            self.groups
        ]
        self.client.find_queue.side_effect = [
            self.queue
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.groups.assert_called_once_with(
            queue=self.queue.id,
            include_deadletter=True)

        self.assertEqual(self.columns_with_dead, columns)
        self.assertEqual(self.data_with_dead, list(data))


class TestDeleteGroup(TestGroup):

    def setUp(self):
        super(TestDeleteGroup, self).setUp()

        self.cmd = group.DeleteGroup(self.app, None)

        self.client.delete_group = mock.Mock()

    def test_delete(self):
        arglist = [
            'queue',
            't1',
        ]

        verifylist = [
            ('queue', 'queue'),
            ('group', ['t1'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_group.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call(queue='queue', group='t1')]

        self.client.delete_group.assert_has_calls(calls)
        self.assertEqual(1, self.client.delete_group.call_count)

    def test_delete_multiple(self):
        arglist = [
            'queue',
            't1',
            't2',
        ]
        verifylist = [
            ('queue', 'queue'),
            ('group', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_group.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(queue='queue', group='t1'),
            mock.call(queue='queue', group='t2')
        ]

        self.client.delete_group.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_group.call_count)


class TestCreateGroup(TestGroup):

    _data = fakes.FakeGroup.create_one()

    columns = ('ID', 'name')

    data = (
        _data.id,
        _data.name,
    )

    def setUp(self):
        super(TestCreateGroup, self).setUp()

        self.cmd = group.CreateGroup(self.app, None)

        self.client.create_group = mock.Mock()

    def test_show_default(self):
        arglist = [
            'queue_id',
            'name',
        ]
        verifylist = [
            ('queue', 'queue_id'),
            ('name', 'name'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_group.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_group.assert_called_with(
            group='name', queue='queue_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Direct Connection Virtual Gateway v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListVirtualGateways(command.Lister):

    _description = _("List of Virtual Gateways.")
    columns = (
        'id',
        'name',
        'vpc id',
        'local ep group id',
        'type',
        'status'
    )

    def get_parser(self, prog_name):
        parser = super(ListVirtualGateways, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_("Specifies the Virtual Gateway ID.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the Virtual Gateway name.")
        )
        parser.add_argument(
            '--vpc_id',
            metavar='<vpc_id>',
            help=_("Specifies the ID of the VPC to be accessed.")
        )
        parser.add_argument(
            '--local_ep_group_id',
            metavar='<local_ep_group_id>',
            help=_("Specifies the ID of the local endpoint group that "
                   "records CIDR blocks of the VPC subnets.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the "
                   "Virtual Gateway.")
        )
        parser.add_argument(
            '--device_id',
            metavar='<device_id>',
            help=_("Specifies the ID of the physical device used by the "
                   "Virtual Gateway.")
        )
        parser.add_argument(
            '--redundant_device_id',
            metavar='<redundant_device_id>',
            help=_("Specifies the ID of the redundant physical device used "
                   "by the Virtual Gateway.")
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_("Specifies the Virtual Gateway type. The value can be "
                   "default or double ipsec.")
        )
        parser.add_argument(
            '--bgp_asn',
            metavar='<bgp_asn>',
            type=int,
            help=_("Specifies the BGP ASN of the Virtual Gateway.")
        )
        parser.add_argument(
            '--ipsec_bandwidth',
            metavar='<ipsec_bandwidth>',
            type=int,
            help=_("Specifies the bandwidth provided for IPsec VPN in Mbit/s.")
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_("Specifies the Virtual Gateway status. "
                   "The value can be ACTIVE, DOWN, BUILD, ERROR, "
                   "PENDING_CREATE, PENDING_UPDATE, or PENDING_DELETE.")
        )
        parser.add_argument(
            '--admin_state_up',
            metavar='<admin_state_up>',
            type=bool,
            help=_("Specifies the administrative status of the Virtual "
                   "Gateway. The value can be true or false.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'id',
            'name',
            'vpc_id',
            'local_ep_group_id',
            'device_id',
            'redundant_device_id',
            'type',
            'bgp_asn',
            'ipsec_bandwidth',
            'status',
            'admin_state_up'
        ]

        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.virtual_gateways(**attrs)

        table = (self.columns, (utils.get_dict_properties(s, self.columns)
                                for s in data))
        return table


class ShowVirtualGateway(command.ShowOne):
    _description = _("Show Virtual Gateway details.")

    def get_parser(self, prog_name):
        parser = super(ShowVirtualGateway, self).get_parser(prog_name)
        parser.add_argument(
            'virtual_gateway',
            metavar='<virtual_gateway>',
            help=_("Specifies the Virtual Gateway ID or name.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas
        obj = client.find_virtual_gateway(parsed_args.virtual_gateway)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateVirtualGateway(command.ShowOne):
    _description = _("Create new Virtual Gateway.")

    def get_parser(self, prog_name):
        parser = super(CreateVirtualGateway, self).get_parser(prog_name)
        parser.add_argument(
            'vpc_id',
            metavar='<vpc_id>',
            help=_("Specifies the ID of the VPC to be accessed.")
        )
        parser.add_argument(
            'local_ep_group_id',
            metavar='<local_ep_group_id>',
            help=_("Specifies the ID of the local endpoint group that records "
                   "CIDR blocks of the VPC subnets.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the Virtual Gateway name.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the "
                   "Virtual Gateway.")
        )
        parser.add_argument(
            '--device_id',
            metavar='<device_id>',
            help=_("Specifies the ID of the physical device used by the "
                   "Virtual Gateway.")
        )
        parser.add_argument(
            '--redundant_device_id',
            metavar='<redundant_device_id>',
            help=_("Specifies the ID of the redundant physical device used by "
                   "the Virtual Gateway.")
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_("Specifies the Virtual Gateway type. "
                   "The value can be default or double ipsec.")
        )
        parser.add_argument(
            '--ipsec_bandwidth',
            metavar='<ipsec_bandwidth>',
            type=int,
            help=_("Specifies the bandwidth provided for IPsec VPN in Mbit/s.")
        )
        parser.add_argument(
            '--bgp_asn',
            metavar='<bgp_asn>',
            type=int,
            help=_("Specifies the BGP ASN of the Virtual Gateway.")
        )
        parser.add_argument(
            '--admin_state_up',
            metavar='<admin_state_up>',
            type=bool,
            help=_("Specifies the administrative status of the "
                   "Virtual Gateway.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'vpc_id',
            'local_ep_group_id',
            'name',
            'description',
            'device_id',
            'redundant_device_id',
            'type',
            'ipsec_bandwidth',
            'bgp_asn',
            'admin_state_up'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_virtual_gateway(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateVirtualGateway(command.ShowOne):
    _description = _("Update a Virtual Gateway.")

    def get_parser(self, prog_name):
        parser = super(UpdateVirtualGateway, self).get_parser(prog_name)
        parser.add_argument(
            'virtual_gateway',
            metavar='<virtual_gateway>',
            help=_("Specifies the Virual Gateway ID or name.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the Virtual Gateway name.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the "
                   "Virtual Gateway.")
        )
        parser.add_argument(
            '--local_ep_group_id',
            metavar='<local_ep_group_id>',
            help=_("Specifies the ID of the local endpoint group that records "
                   "CIDR blocks of the VPC subnets.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'name',
            'description',
            'local_ep_group_id'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        if parsed_args.virtual_gateway:
            virtual_gateway = client.find_virtual_gateway(
                parsed_args.virtual_gateway
            )
            obj = client.update_virtual_gateway(
                virtual_gateway.id, **attrs
            )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteVirtualGateway(command.Command):
    _description = _("Delete the Virtual Gateway.")

    def get_parser(self, prog_name):
        parser = super(DeleteVirtualGateway, self).get_parser(prog_name)
        parser.add_argument(
            'virtual_gateway',
            metavar='<virtual_gateway>',
            help=_("Virtual Gateway to delete.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas
        if parsed_args.virtual_gateway:
            virtual_gateway = client.find_virtual_gateway(
                parsed_args.virtual_gateway)
            client.delete_virtual_gateway(virtual_gateway.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from otcextensions.sdk.modelartsv1.v1.training_job import ConfigSpec


class TrainingJobConfig(ConfigSpec):
    base_path = "/training-job-configs"

    resources_key = "configs"

    _query_mapping = resource.QueryParameters(
        "config_type",
        "limit",
        "order",
        "offset",
        "search_content",
        "sort_by",
        limit="per_page",
        offset="page",
        sort_by="sortBy",
    )

    # capabilities
    allow_create = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True
    allow_list = True

    #: Name of a training job configuration.
    config_name = resource.Body("config_name")
    #: Description of a training job configuration.
    config_desc = resource.Body("config_desc")
    #: Name of a training job configuration.
    name = resource.Body("name", alias="config_name")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from otcextensions.tests.functional.osclient.vpc.v1 import common


class TestPeering(common.VpcTestCase):
    """Functional Tests for NAT Gateway"""

    def setUp(self):
        super(TestPeering, self).setUp()

    def test_vpc_peering_list(self):
        json_output = json.loads(self.openstack(
            'vpc peering list -f json '
        ))
        self.assertIsNotNone(json_output)

    def test_vpc_peering_list_filters(self):
        json_output = json.loads(self.openstack(
            'vpc peering list -f json '
            '--limit 1 '
            '--id 2 '
            '--name 3 '
            '--project-id 4 '
            '--router-id 5 '
            '--status ACTIVE '
        ))
        self.assertIsNotNone(json_output)

    def test_vpc_peering(self):
        self.addCleanup(self.delete_vpc_peering)
        peering = self.create_vpc_peering()
        peering_id = peering['id']
        peering_name = peering['name']
        local_router_id = peering['local_vpc_info']['router_id']
        peer_router_id = peering['peer_vpc_info']['router_id']

        # List Vpc Peering By Id
        json_output = json.loads(self.openstack(
            'vpc peering list -f json '
            '--id {}'.format(peering_id)
        ))
        self.assertEqual(json_output[0]['Name'], peering_name)
        self.assertEqual(json_output[0]['Id'], peering_id)

        # List Vpc Peering By Name
        json_output = json.loads(self.openstack(
            'vpc peering list -f json '
            '--name {}'.format(peering_name)
        ))
        self.assertEqual(json_output[0]['Name'], peering_name)
        self.assertEqual(json_output[0]['Id'], peering_id)

        # List Vpc Peering by Requester Router ID
        json_output = json.loads(self.openstack(
            'vpc peering list -f json '
            '--router-id {}'.format(local_router_id)
        ))
        self.assertIsNotNone(json_output)

        # List Vpc Peering by Accepter Router ID
        json_output = json.loads(self.openstack(
            'vpc peering list -f json '
            '--router-id {}'.format(peer_router_id)
        ))
        for peering in json_output:
            self.assertEqual(
                peering['Peer Router Id'],
                peer_router_id
            )

        # Show Vpc Peering by Name
        json_output = json.loads(self.openstack(
            'vpc peering show -f json ' + peering_name
        ))
        self.assertEqual(json_output['name'], peering_name)
        self.assertEqual(json_output['id'], peering_id)

        # Show Vpc Peering by Id
        json_output = json.loads(self.openstack(
            'vpc peering show -f json ' + peering_id
        ))
        self.assertEqual(json_output['name'], peering_name)
        self.assertEqual(json_output['id'], peering_id)

        # Update Vpc Peering
        peering_name = peering_name + "-updated"
        description = "test vpc peering updated by otce cli"
        json_output = json.loads(self.openstack(
            'vpc peering update {peering_id} '
            '--name {name} '
            '--description "{desc}" '
            '-f json'.format(
                peering_id=peering_id,
                name=peering_name,
                desc=description)
        ))
        self.assertEqual(json_output['name'], peering_name)
        self.assertEqual(json_output['description'], description)
